# Copyright (C) 2009-2010 Red Hat Inc.
# Author: Kushal Das <kushal@redhat.com>
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
# the full text of the license.

import os
import sys
import subprocess
import xmlrpclib

from rpmUtils.miscutils import splitFilename
from bugzilla import RHBugzilla3

URL = 'https://bugzilla.redhat.com/xmlrpc.cgi'


class BoogReporter:
    """
    Backend for boog
    """

    def __init__(self):
        """
        The  class
        """
        getout = False
        if os.path.exists('/etc/fedora-release'):
            data = open('/etc/fedora-release').readline()[:-1]
            if data == 'Fedora release 13 (Goddard)':
                self.fedorarelease = '13'
            elif data == 'Fedora release 12 (Constantine)':
                self.fedorarelease = '12'
            elif data == 'Fedora release 11 (Leonidas)':
                self.fedorarelease = '11'
            elif data == 'Fedora release 10 (Cambridge)':
                self.fedorarelease = '10'
            elif data.find('Rawhide') != -1:
                self.fedorarelease = 'rawhide'
            else:
                self.fedorarelease = 'rawhide'
        else:
            getout = True

        if getout:
            print "Sorry, but this version of Fedora is not yet supported.\
                    Please contact the upstream"
            sys.exit(0)

        self.bz = RHBugzilla3(url=URL)
        self.bug = None
        self.init()


    def login(self, username, password):
        return self.bz.login(username, password)

    def init(self):
        """
        Init all information fields
        """
        self.name = ''
        self.srpm = ''
        self.rpm_details = ''
        self.summary = ''
        self.user_desc = ''
        self.auto_desc = ''
        self.arch = ''
        self.sestatus = ''
        self.smolt_profile = ''
        self.lspci_details = ''
        self.cpuinfo_details = ''
        self.yum_information = ''
        self.dracut_information = ''
        self.virsh_capabilities_details = ''
        self.files_to_upload = []
        self.product_list = None

    def file_a_new_bug(self):
        """
        Just file a new bug with all details
        """
        if self.srpm:
            self.bug = self.bz.createbug(product='Fedora', component=self.srpm, \
                          version=self.fedorarelease, summary=self.summary, op_sys='Linux',\
                          platform = self.arch, description=self.user_desc + self.auto_desc)

            for file_name in self.files_to_upload:
                try:
                    self.bz.attachfile(str(self.bug.id), file_name, file_name, contenttype='text/plain')
                except:
                    continue


    def get_product_list(self):
        """
        Get all components names from Bugzilla
        """
        self.product_list = self.bz.getcomponents('Fedora')

    def get_arch(self):
        """
        Get the system architecture
        """
        self.arch = os.uname()[-1]

    def get_lspci(self):
        """
        Get the lscpi -vvv output
        """
        cmd = 'lspci -vvv'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        self.lspci_details = '\n$lspci -vvv\n' + stdout.read()

    def get_cpuinfo(self):
        """
        Get the /proc/cpuinfo
        """
        cmd = 'cat /proc/cpuinfo'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        self.cpuinfo_details = '\n$cat /proc/cpuinfo\n' + stdout.read()

    def get_virsh_capabilities(self):
        """
        Get the output of virsh capabilities
        """
        cmd = 'virsh capabilities'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        self.virsh_capabilities_details = '\n$virsh capabilities\n' + stdout.read()

    def get_dracut_info(self):
        """
        Get all information related dracut.
        Based on -
        https://fedoraproject.org/wiki/How_to_debug_Dracut_problems
        """
        self.files_to_upload.append('/etc/fstab')

        cmd = 'cat /proc/cmdline'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        info = stdout.read()

        cmd = 'dmsetup ls --tree'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        info += '\n\n$dmsetup ls --tree\n' + stdout.read()
        del stdout

        cmd = 'blkid'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        info += '\n$blkid\n' + stdout.read()
        del stdout

        if os.path.exists('/etc/dracut.conf'):
            self.files_to_upload.append('/etc/dracut.conf')

        cmd = 'lvm pvdisplay'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout
        info += '\n$lvm pvdisplay\n' + stdout.read()
        del stdout

        cmd = 'lvm vgdisplay'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout
        info += '\n$lvm vgdisplay\n' + stdout.read()

        cmd = 'lvm lvdisplay'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout
        info += '\n$lvm lvdisplay\n' + stdout.read()

        cmd = 'cat /proc/mdstat'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        info += '\n$cat /proc/mdstat\n' + stdout.read()

        cmd = 'ip addr show'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        info += '\n$ip addr show\n' + stdout.read()

        self.dracut_information = info



    def set_summary(self, summary):
        """
        Set the summary of the new bug going to be created
        """
        self.summary = summary

    def set_description(self, desc):
        """
        Set the description of the new bug going to be created
        """
        self.user_desc = desc



    def get_rpm_name_for_file(self,filepath):
        """
        On a given filepath this will return the actual RPM name
        """
        cmd = 'rpm -q  -f ' + filepath
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        data = stdout.read()
        return data.strip()

    def get_srpm_name_for_package(self):
        """
        Find the srpm name for the given package
        """
        cmd = 'rpm -q --qf "%{SOURCERPM}" ' + self.name
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        data = stdout.read()
        data = data.strip()
        self.srpm = splitFilename(data)[0]
        return self.srpm

    def get_srpm_name_for_file(self,filepath):
        """
        On a given filepath, this will return SRPM name
        """
        cmd = 'rpm -q --qf "%{SOURCERPM}\n" -f ' + filepath
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        data = stdout.read()
        data = data.strip()
        self.srpm = splitFilename(data)[0]
        return self.srpm


    def ask_app_name(self):
        self.name = raw_input("Name of the application where you are having problem: ")

    def set_name(self,name):
        self.name = name

    def get_sestatus(self):
        """
        Find the SELINUX details of the system
        """
        cmd = '/usr/sbin/sestatus'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        self.sestatus = stdout.read()

    def get_application_details(self):
        """
        This function will find the rpm details
        """
        cmd = 'rpm -qa ' + self.name
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        data = stdout.read().strip()
        if data == '':
            return False
        if data.find('\n') != -1:
            return data.split('\n')
        self.rpm_details = data.strip()
        return True

    def get_yum_details(self):
        """
        This information is needed for yum bugs.
        skvidal said , this is a good point to start
        """
        cmd = 'yum --version'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        self.yum_information = stdout.read()

    def get_smolt_profile(self):
        """
        To get smolt profile of the system
        """
        cmd = 'smoltSendProfile -p'
        stdout = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout
        self.smolt_profile = '\n$smoltSendProfile -p\n' + stdout.read()

    def get_uname(self):
        """
        Get the uname details
        """
        self.uname_details = " ".join(os.uname())

    def get_selinux_details(self):
        """
        https://fedoraproject.org/wiki/Bug_info_SELinux
        """
        self.files_to_upload.append('/var/log/messages')
        if os.path.exists('/var/log/audit/audit.log'):
            self.files_to_upload.append('/var/log/audit/audit.log')

    def get_additional_info(self):
        """
        Get the other details according to the srpm
        For now we are trying to follow -
        https://fedoraproject.org/wiki/BugsAndFeatureRequests#Information_required_for_bugs_in_specific_components
        We have to improve this for different packages
        """
        if self.name.find('selinux-policy') != -1:
            self.get_selinux_details()
        elif self.name in ['python-virtinst', 'qemu-kvm', 'virt-manager',
            'virt-viewer','virt-top']:
            self.get_uname()
            self.get_cpuinfo()
            self.get_lspci()
            self.get_virsh_capabilities()
        elif self.name.find('xorg') != -1:
            for logfile in glob.glob('/var/log/Xorg.*.log'):
                self.files_to_upload.append(logfile)
            self.get_smolt_profile()
            self.files_to_upload.append('/var/log/messages')
            if os.path.exists('/etc/X11/xorg.conf'):
                self.files_to_upload.append('/etc/X11/xorg.conf')
        elif self.name.startswith('dracut'):
            self.get_dracut_info()


    def create_final_description(self):
        """
        This will create the final description for submitting to bugzilla
        """
        #self.ask_app_name()
        #details = self.get_application_details()
        self.get_srpm_name_for_package()
        self.get_additional_info()
        self.get_yum_details()
        self.get_arch()
        self.get_sestatus()

        self.auto_desc = '\n\nPackage Version: ' + self.rpm_details + \
                         '\n\nAdditional Information:\n' + self.sestatus + \
                         '\n\nYum Information:\n' + self.yum_information

        self.auto_desc += '\n'.join([self.smolt_profile, self.lspci_details,\
                          self.cpuinfo_details, self.dracut_information, \
                          self.virsh_capabilities_details])

        self.auto_desc +=  '\nThe following files will be attached (if any):\n'\
                            + '\n'.join(set(self.files_to_upload))


if __name__ == '__main__':
    b = BoogReporter('username','password')
    b.get_product_list()
    from pprint import pprint
    pprint(b.product_list)
